@@ -42,8 +42,8 @@ from mapproxy.grid import MetaGrid
 from mapproxy.image import merge_images
 from mapproxy.image.tile import TileSplitter
 from mapproxy.layer import MapQuery, BlankImage
-from mapproxy.util import ThreadedExecutor
 from mapproxy.config import base_config
+from mapproxy.util import async
 
 import logging
 log = logging.getLogger(__name__)
@@ -61,9 +61,9 @@ class TileManager(object):
         self.minimize_meta_requests = minimize_meta_requests
         self._expire_timestamp = None
         self.transparent = self.sources[0].transparent
-        self.thread_pool_size = base_config().cache.concurrent_tile_creators
+        self.concurrent_tile_creators = base_config().cache.concurrent_tile_creators
         
-        if meta_buffer is not None or meta_size:
+        if meta_buffer or (meta_size and not meta_size == [1, 1]):
             if all(source.supports_meta_tiles for source in sources):
                 self.meta_grid = MetaGrid(grid, meta_size=meta_size, meta_buffer=meta_buffer)
             elif any(source.supports_meta_tiles for source in sources):
@@ -80,7 +80,7 @@ class TileManager(object):
             # TODO cache eviction
             if self.is_cached(tile):
                 self.cache.load(tile, with_metadata)
-            else:
+            elif tile.coord is not None:
                 uncached_tiles.append(tile)
         
         if uncached_tiles:
@@ -92,6 +92,11 @@ class TileManager(object):
         
         return tiles
     
+    def remove_tile_coords(self, tile_coords):
+        tiles = TileCollection(tile_coords)
+        for tile in tiles:
+            self.cache.remove(tile)
+    
     def creator(self):
         return TileCreator(self.cache, self.sources, self.grid, self.meta_grid, self)
     
@@ -106,14 +111,28 @@ class TileManager(object):
         """
         if isinstance(tile, tuple):
             tile = Tile(tile)
-        max_mtime = self.expire_timestamp(tile)
         cached = self.cache.is_cached(tile)
+        max_mtime = self.expire_timestamp(tile)
         if cached and max_mtime is not None:
             stale = self.cache.timestamp_created(tile) < max_mtime
             if stale:
                 cached = False
         return cached
     
+    def is_stale(self, tile):
+        """
+        Return True if tile exists _and_ is expired.
+        """
+        if isinstance(tile, tuple):
+            tile = Tile(tile)
+        if self.cache.is_cached(tile):
+            # tile exists
+            if not self.is_cached(tile):
+                # expired
+                return True
+            return False
+        return False
+    
     def expire_timestamp(self, tile=None):
         """
         Return the timestamp until which a tile should be accepted as up-to-date,
@@ -158,7 +177,7 @@ class TileCreator(object):
         return created_tiles
 
     def _create_single_tiles(self, tiles):
-        if self.tile_mgr.thread_pool_size > 1 and len(tiles) > 1:
+        if self.tile_mgr.concurrent_tile_creators > 1 and len(tiles) > 1:
             return self._create_threaded(self._create_single_tile, tiles)
         
         created_tiles = []
@@ -167,9 +186,9 @@ class TileCreator(object):
         return created_tiles
     
     def _create_threaded(self, create_func, tiles):
-        pool = ThreadedExecutor(create_func, pool_size=self.tile_mgr.thread_pool_size)
         result = []
-        for new_tiles in pool.execute(tiles):
+        async_pool = async.Pool(self.tile_mgr.concurrent_tile_creators)
+        for new_tiles in async_pool.imap(create_func, tiles):
             result.extend(new_tiles)
         return result
     
@@ -198,20 +217,24 @@ class TileCreator(object):
             except BlankImage:
                 return None
         
-        imgs = []
-        for source in self.sources:
+        def get_map_from_source(source):
             try:
                 img = source.get_map(query)
             except BlankImage:
-                pass
+                return None
             else:
+                return img
+        
+        imgs = []
+        for img in async.imap(get_map_from_source, self.sources):
+            if img is not None:
                 imgs.append(img)
         
         if not imgs: return None
-        return merge_images(imgs)
+        return merge_images(imgs, size=query.size)
     
     def _create_meta_tiles(self, meta_tiles):
-        if self.tile_mgr.thread_pool_size > 1 and len(meta_tiles) > 1:
+        if self.tile_mgr.concurrent_tile_creators > 1 and len(meta_tiles) > 1:
             return self._create_threaded(self._create_meta_tile, meta_tiles)
         
         created_tiles = []
