@@ -26,6 +26,7 @@ from mapproxy.client.http import HTTPClient
 from mapproxy.srs import make_lin_transf, SRS
 from mapproxy.image import ImageSource
 from mapproxy.image.transform import ImageTransformer
+from mapproxy.featureinfo import create_featureinfo_doc
 
 import logging
 log = logging.getLogger(__name__)
@@ -143,7 +144,27 @@ class WMSClient(object):
         req.params.srs = query.srs.srs_code
         req.params.format = format
         return req
-
+    
+    def combined_client(self, other, query):
+        """
+        Return a new WMSClient that combines this request with the `other`. Returns
+        ``None`` if the clients are not combinable (e.g. different URLs).
+        """
+        if self.request_template.url != other.request_template.url:
+            return None
+        
+        if self.supported_srs != other.supported_srs:
+          return None
+        
+        if self.supported_formats != other.supported_formats:
+          return None
+        
+        new_req = self.request_template.copy()
+        new_req.params.layers = new_req.params.layers + other.request_template.params.layers
+        
+        return WMSClient(new_req, http_client=self.http_client, http_method=self.http_method,
+                         resampling=self.resampling)
+        
 
 class WMSInfoClient(object):
     def __init__(self, request_template, supported_srs=None, http_client=None):
@@ -155,11 +176,17 @@ class WMSInfoClient(object):
     
     def get_info(self, query):
         if self.supported_srs and query.srs not in self.supported_srs:
-            return self._get_transformed(query)
+            query = self._get_transformed_query(query)
         resp = self._retrieve(query)
-        return resp
+        info_format = resp.headers.get('Content-type', None)
+        if not info_format:
+            info_format = query.info_format
+        return create_featureinfo_doc(resp.read(), info_format)
     
-    def _get_transformed(self, query):
+    def _get_transformed_query(self, query):
+        """
+        Handle FI requests for unsupported SRS.
+        """
         req_srs = query.srs
         req_bbox = query.bbox
         info_srs = self._best_supported_srs(req_srs)
@@ -171,9 +198,10 @@ class WMSInfoClient(object):
         info_pos = make_lin_transf((info_bbox), (0, query.size[1], query.size[0], 0))(info_coord)
         
         info_query = InfoQuery(info_bbox, query.size, info_srs, info_pos, query.info_format)
-        return self._retrieve(info_query)
+        return info_query
     
     def _best_supported_srs(self, srs):
+        # always choose the first, distortion should not matter
         return self.supported_srs[0]
     
     def _retrieve(self, query):
@@ -185,9 +213,8 @@ class WMSInfoClient(object):
         req.params.bbox = query.bbox
         req.params.size = query.size
         req.params.pos = query.pos
-        # del req.params['info_format']
         req.params['query_layers'] = req.params['layers']
-        if query.info_format:
+        if not 'info_format' in req.params and query.info_format:
             req.params['info_format'] = query.info_format
         if not req.params.format:
             req.params.format = query.format or 'image/png'
@@ -222,4 +249,31 @@ class WMSLegendClient(object):
             req.params.format = query.format or 'image/png'
         if query.scale:
             req.params['scale'] = query.scale
-        return req.complete_url
\ No newline at end of file
+        return req.complete_url
+    
+    @property
+    def identifier(self):
+        return (self.request_template.url, self.request_template.params.layer)
+        
+class WMSLegendURLClient(object):
+    def __init__(self, static_url, http_client=None):
+        self.url = static_url
+        self.http_client = http_client or HTTPClient()
+    
+    def get_legend(self, query):
+        resp = self.http_client.open(self.url)
+        format = split_mime_type(query.format)[1]
+        self._check_resp(resp)
+        return ImageSource(resp, format=format)
+    
+    def _check_resp(self, resp):
+        if 'Content-type' not in resp.headers:
+            raise SourceError('response from source WMS has no Content-type header')
+        if not resp.headers['Content-type'].startswith('image/'):
+            raise SourceError('no image returned from static LegendURL')
+    
+    @property
+    def identifier(self):
+        return (self.url, None)
+        
+        
\ No newline at end of file
