@@ -22,6 +22,7 @@ wms = dict(
     strict = False,
     request_parser = 'default',
     client_request = 'default',
+    concurrent_layer_renderer = 1,
 )
 debug_mode = False
 
@@ -41,6 +42,7 @@ image = dict(
     stretch_factor = 1.15,
     max_shrink_factor = 4.0,
     paletted = True,
+    transparent_color_tolerance = 5,
 )
 # number of concurrent requests to a tile source
 
