@@ -24,14 +24,13 @@ import hashlib
 import urlparse
 import yaml #pylint: disable-msg=F0401
 from copy import deepcopy
-from functools import wraps
 
 import logging
 log = logging.getLogger(__name__)
 
 from mapproxy.srs import SRS
 from mapproxy.util.ext.odict import odict
-from mapproxy.cache.file import FileCache
+from mapproxy.cache.file import FileCache, DummyCache
 from mapproxy.util.lock import SemLock
 from mapproxy.config.config import load_default_config
 from mapproxy.client.http import auth_data_from_url, HTTPClient
@@ -75,7 +74,7 @@ from mapproxy.layer import (
     ResolutionConditional, map_extent_from_grid, merge_layer_extents
 )
 from mapproxy.client.tile import TileClient, TileURLTemplate
-from mapproxy.client.wms import WMSClient, WMSInfoClient, WMSLegendClient
+from mapproxy.client.wms import WMSClient, WMSInfoClient, WMSLegendClient, WMSLegendURLClient
 from mapproxy.service.wms import WMSServer, WMSLayer, WMSGroupLayer
 from mapproxy.service.tile import TileServer, TileLayer
 from mapproxy.service.kml import KMLServer
@@ -85,8 +84,9 @@ from mapproxy.source.wms import WMSSource, WMSInfoSource, WMSLegendSource
 from mapproxy.source.tile import TiledSource
 from mapproxy.cache.tile import TileManager
 from mapproxy.cache.legend import LegendCache
-from mapproxy.util import local_base_config
+from mapproxy.util import local_base_config, memoize
 from mapproxy.config.coverage import load_coverage
+from mapproxy.featureinfo import XSLTransformer, has_xslt_support
 
 class ConfigurationError(Exception):
     pass
@@ -210,11 +210,15 @@ class ProxyConfiguration(object):
             if isinstance(layers_conf[0], dict) and len(layers_conf[0].keys()) == 1:
                 # looks like ordered legacy config
                 layers_conf = self._legacy_layers_conf_dict()
+            elif len(layers_conf) == 1 and 'layers' in layers_conf[0]:
+                # single root layer in list -> remove list
+                layers_conf = layers_conf[0]
             else:
-                # wrap in root layer
+                # layer list without root -> wrap in root layer
                 layers_conf = dict(title=None, layers=layers_conf)
         
-        if 'layers' not in layers_conf:
+        if len(set(layers_conf.keys()) &
+               set('layers name title sources'.split())) < 2:
             # looks like unordered legacy config
             layers_conf = self._legacy_layers_conf_dict()
         
@@ -255,17 +259,6 @@ class ProxyConfiguration(object):
     def base_config(self):
         return self.globals.base_config
 
-def memoize(func):
-    @wraps(func)
-    def wrapper(*args):
-        if not hasattr(func, '__memoize_cache'):
-            func.__memoize_cache = {}
-        key = args
-        if key not in func.__memoize_cache:
-            func.__memoize_cache[key] = func(*args)
-        return func.__memoize_cache[key]
-    return wrapper
-
 def list_of_dicts_to_ordered_dict(dictlist):
     """
     >>> d = list_of_dicts_to_ordered_dict([{'a': 1}, {'b': 2}, {'c': 3}])
@@ -393,9 +386,13 @@ class GlobalConfiguration(ConfigurationBase):
     def get_path(self, key, local, global_key=None, default_key=None):
         value = self.get_value(key, local, global_key, default_key)
         if value is not None:
-            value = os.path.join(self.base_config.conf_base_dir, value)
+            value = self.abspath(value)
         return value
     
+    def abspath(self, path):
+        return os.path.join(self.base_config.conf_base_dir, path)
+        
+    
 
 def dotted_dict_get(key, d):
     """
@@ -414,7 +411,7 @@ def dotted_dict_get(key, d):
     if parts: # not completely resolved
         return None
     return d
-    
+
 class SourceConfiguration(ConfigurationBase):
     @classmethod
     def load(cls, conf, context):
@@ -429,6 +426,22 @@ class SourceConfiguration(ConfigurationBase):
     def coverage(self):
         if not 'coverage' in self.conf: return None
         return load_coverage(self.conf['coverage'])
+    
+    def http_client(self, url):
+        http_client = None
+        url, (username, password) = auth_data_from_url(url)
+        insecure = ssl_ca_certs = None
+        if 'https' in url:
+            insecure = self.context.globals.get_value('http.ssl_no_cert_checks', self.conf)
+            ssl_ca_certs = self.context.globals.get_path('http.ssl_ca_certs', self.conf)
+        
+        timeout = self.context.globals.get_value('http.client_timeout', self.conf)
+        headers = self.context.globals.get_value('http.headers', self.conf)
+        
+        http_client = HTTPClient(url, username, password, insecure=insecure,
+                                 ssl_ca_certs=ssl_ca_certs, timeout=timeout,
+                                 headers=headers)
+        return http_client, url
 
 def resolution_range(conf):
     if 'min_res' in conf or 'max_res' in conf:
@@ -446,22 +459,31 @@ class WMSSourceConfiguration(SourceConfiguration):
         min_res max_res min_scale max_scale'''.split())
     required_keys = set('req'.split())
     
-    def http_client(self, request):
-        http_client = None
-        url, (username, password) = auth_data_from_url(request.url)
-        if username and password:
-            request.url = url
-        insecure = ssl_ca_certs = None
-        if 'https' in url:
-            insecure = self.context.globals.get_value('http.ssl_no_cert_checks', self.conf)
-            ssl_ca_certs = self.context.globals.get_path('http.ssl_ca_certs', self.conf)
-        
-        timeout = self.context.globals.get_value('http.client_timeout', self.conf)
-        http_client = HTTPClient(url, username, password, insecure=insecure,
-                                 ssl_ca_certs=ssl_ca_certs, timeout=timeout)
-        return http_client
+    @staticmethod
+    def static_legend_source(url, context):
+        cache_dir = os.path.join(context.globals.get_path('cache.base_dir', {}),
+                                 'legends')
+        if url.startswith('file://') and not url.startswith('file:///'):
+            prefix = 'file://'
+            url = prefix + context.globals.abspath(url[7:])
+        lg_client = WMSLegendURLClient(url)
+        legend_cache = LegendCache(cache_dir=cache_dir)
+        return WMSLegendSource([lg_client], legend_cache)
+    
+    def fi_xslt_transformer(self, conf, context):
+        fi_transformer = None
+        fi_xslt = conf.get('featureinfo_xslt')
+        if fi_xslt:
+            if not has_xslt_support:
+                raise ValueError('featureinfo_xslt requires lxml. Please install.')
+            fi_xslt = context.globals.abspath(fi_xslt)
+            fi_transformer = XSLTransformer(fi_xslt)
+        return fi_transformer
     
     def source(self, params=None):
+        if not self.conf.get('wms_opts', {}).get('map', True):
+            return None
+        
         if not self.context.seed and self.conf.get('seed_only'):
             return DummySource()
         
@@ -474,6 +496,8 @@ class WMSSourceConfiguration(SourceConfiguration):
         transparent = self.conf['req'].get('transparent', 'false')
         transparent = bool(str(transparent).lower() == 'true')
         
+        opacity = self.conf.get('image', {}).get('opacity')
+        
         resampling = self.context.globals.get_value('image.resampling_method', self.conf)
         
         supported_srs = [SRS(code) for code in self.conf.get('supported_srs', [])]
@@ -493,15 +517,23 @@ class WMSSourceConfiguration(SourceConfiguration):
         coverage = self.coverage()
         res_range = resolution_range(self.conf)
         
+        transparent_color = self.conf.get('image', {}).get('transparent_color')
+        transparent_color_tolerance = self.context.globals.get_value(
+            'image.transparent_color_tolerance', self.conf)
+        if transparent_color:
+            transparent_color = parse_color(transparent_color)
+        
         http_method = self.context.globals.get_value('http.method', self.conf)
         
         request = create_request(self.conf['req'], params, version=version)
-        http_client = self.http_client(request)
+        http_client, request.url = self.http_client(request.url)
         client = WMSClient(request, supported_srs, http_client=http_client, 
                            http_method=http_method, resampling=resampling, lock=lock,
                            supported_formats=supported_formats or None)
         return WMSSource(client, transparent=transparent, coverage=coverage,
-                         res_range=res_range)
+                         res_range=res_range, opacity=opacity,
+                         transparent_color=transparent_color,
+                         transparent_color_tolerance=transparent_color_tolerance)
     
     def fi_source(self, params=None):
         if params is None: params = {}
@@ -511,11 +543,20 @@ class WMSSourceConfiguration(SourceConfiguration):
         supported_srs = [SRS(code) for code in self.conf.get('supported_srs', [])]
         fi_source = None
         if self.conf.get('wms_opts', {}).get('featureinfo', False):
-            version = self.conf.get('wms_opts', {}).get('version', '1.1.1')
+            wms_opts = self.conf['wms_opts']
+            version = wms_opts.get('version', '1.1.1')
+            if 'featureinfo_format' in wms_opts:
+                params['info_format'] = wms_opts['featureinfo_format']
             fi_request = create_request(self.conf['req'], params,
                 req_type='featureinfo', version=version)
-            fi_client = WMSInfoClient(fi_request, supported_srs=supported_srs)
-            fi_source = WMSInfoSource(fi_client)
+            
+            fi_transformer = self.fi_xslt_transformer(self.conf.get('wms_opts', {}),
+                                                     self.context)
+            
+            http_client, fi_request.url = self.http_client(fi_request.url)
+            fi_client = WMSInfoClient(fi_request, supported_srs=supported_srs,
+                                      http_client=http_client)
+            fi_source = WMSInfoSource(fi_client, fi_transformer=fi_transformer)
         return fi_source
     
     def lg_source(self, params=None):
@@ -526,7 +567,11 @@ class WMSSourceConfiguration(SourceConfiguration):
         lg_source = None
         cache_dir = os.path.join(self.context.globals.get_path('cache.base_dir', {}),
                                  'legends')
-        if self.conf.get('wms_opts', {}).get('legendgraphic', False):
+                                 
+        if self.conf.get('wms_opts', {}).get('legendurl', False):
+            lg_url = self.conf.get('wms_opts', {}).get('legendurl')
+            lg_source = WMSSourceConfiguration.static_legend_source(lg_url, self.context)
+        elif self.conf.get('wms_opts', {}).get('legendgraphic', False):
             version = self.conf.get('wms_opts', {}).get('version', '1.1.1')
             lg_req = self.conf['req'].copy()
             lg_clients = []
@@ -536,7 +581,8 @@ class WMSSourceConfiguration(SourceConfiguration):
                 lg_req['layer'] = lg_layer
                 lg_request = create_request(lg_req, params,
                     req_type='legendgraphic', version=version)
-                lg_client = WMSLegendClient(lg_request)
+                http_client, lg_request.url = self.http_client(lg_request.url)
+                lg_client = WMSLegendClient(lg_request, http_client=http_client)
                 lg_clients.append(lg_client)
             legend_cache = LegendCache(cache_dir=cache_dir)
             lg_source = WMSLegendSource(lg_clients, legend_cache)
@@ -545,7 +591,8 @@ class WMSSourceConfiguration(SourceConfiguration):
 
 class TileSourceConfiguration(SourceConfiguration):
     source_type = ('tile',)
-    optional_keys = set('''type grid request_format origin coverage seed_only'''.split())
+    optional_keys = set('''type grid request_format origin coverage seed_only
+                           transparent http'''.split())
     required_keys = set('url'.split())
     defaults = {'origin': 'sw', 'grid': 'GLOBAL_MERCATOR'}
     
@@ -562,13 +609,17 @@ class TileSourceConfiguration(SourceConfiguration):
             origin = 'sw'
             # TODO raise some configuration exception
         
+        http_client, url = self.http_client(url)
         grid = self.context.grids[self.conf['grid']].tile_grid()
         coverage = self.coverage()
+        opacity = self.conf.get('image', {}).get('opacity')
+        transparent = self.conf.get('transparent')
         
         inverse = True if origin == 'nw' else False
         format = file_ext(params['format'])
-        client = TileClient(TileURLTemplate(url, format=format))
-        return TiledSource(grid, client, inverse=inverse, coverage=coverage)
+        client = TileClient(TileURLTemplate(url, format=format), http_client=http_client)
+        return TiledSource(grid, client, inverse=inverse, coverage=coverage, opacity=opacity,
+                           transparent=transparent)
 
 def file_ext(mimetype):
     _mime_class, format, _options = split_mime_type(mimetype)
@@ -584,7 +635,7 @@ class DebugSourceConfiguration(SourceConfiguration):
 class CacheConfiguration(ConfigurationBase):
     optional_keys = set('''format cache_dir grids link_single_color_images image
         use_direct_from_res use_direct_from_level meta_buffer meta_size
-        minimize_meta_requests'''.split())
+        minimize_meta_requests disable_storage'''.split())
     optional_keys.update(tile_filter_conf_keys)
     required_keys = set('name sources'.split())
     defaults = {'format': 'image/png', 'grids': ['GLOBAL_MERCATOR']}
@@ -594,6 +645,9 @@ class CacheConfiguration(ConfigurationBase):
             global_key='cache.base_dir')
         
     def _file_cache(self, grid_conf):
+        if self.conf.get('disable_storage', False):
+            return DummyCache()
+        
         cache_dir = self.cache_dir()
         grid_conf.tile_grid() #create to resolve `base` in grid_conf.conf
         suffix = grid_conf.conf['srs'].replace(':', '')
@@ -629,7 +683,9 @@ class CacheConfiguration(ConfigurationBase):
             sources = []
             for source_conf in [self.context.sources[s] for s in self.conf['sources']]:
                 source = source_conf.source({'format': request_format})
-                sources.append(source)
+                if source:
+                    sources.append(source)
+            assert sources, 'no sources configured for %s' % self.conf['name']
             cache = self._file_cache(grid_conf)
             tile_grid = grid_conf.tile_grid()
             mgr = TileManager(tile_grid, cache, sources, file_ext(request_format),
@@ -644,18 +700,20 @@ class CacheConfiguration(ConfigurationBase):
     @memoize
     def map_layer(self):
         resampling = self.context.globals.get_value('image.resampling_method', self.conf)
-        
+        opacity = self.conf.get('image', {}).get('opacity')
         caches = []
         main_grid = None
         for grid, extent, tile_manager in self.caches():
             if main_grid is None:
                 main_grid = grid
-            caches.append((CacheMapLayer(tile_manager, extent=extent, resampling=resampling), (grid.srs,)))
+            caches.append((CacheMapLayer(tile_manager, extent=extent, resampling=resampling,
+                                         opacity=opacity),
+                          (grid.srs,)))
         
         if len(caches) == 1:
             layer = caches[0][0]
         else:
-            layer = SRSConditional(caches, caches[0][0].extent, caches[0][0].transparent)
+            layer = SRSConditional(caches, caches[0][0].extent, caches[0][0].transparent, opacity=opacity)
         
         if 'use_direct_from_level' in self.conf:
             self.conf['use_direct_from_res'] = main_grid.resolution(self.conf['use_direct_from_level'])
@@ -663,14 +721,14 @@ class CacheConfiguration(ConfigurationBase):
             if len(self.conf['sources']) != 1:
                 raise ValueError('use_direct_from_level/res only supports single sources')
             source_conf = self.context.sources[self.conf['sources'][0]]
-            layer = ResolutionConditional(layer, source_conf.source(), self.conf['use_direct_from_res'], main_grid.srs, layer.extent)
+            layer = ResolutionConditional(layer, source_conf.source(), self.conf['use_direct_from_res'],
+                                          main_grid.srs, layer.extent, opacity=opacity)
         return layer
 
 
-
 class WMSLayerConfiguration(ConfigurationBase):
     optional_keys = set('name layers min_res max_res min_scale '
-                        'max_scale sources'.split())
+                        'max_scale legendurl sources'.split())
     required_keys = set('title'.split())
     
     @memoize
@@ -683,7 +741,7 @@ class WMSLayerConfiguration(ConfigurationBase):
             for layer_conf in layers_conf:
                 layers.append(WMSLayerConfiguration(layer_conf, self.context).wms_layer())
         
-        if 'sources' in self.conf:
+        if 'sources' in self.conf or 'legendurl' in self.conf:
             this_layer = LayerConfiguration(self.conf, self.context).wms_layer()
         
         if not layers and not this_layer:
@@ -692,19 +750,27 @@ class WMSLayerConfiguration(ConfigurationBase):
         if not layers:
             layer = this_layer
         else:
-            layer = WMSGroupLayer({'title':self.conf['title'], 'name': self.conf.get('name')},
+            layer = WMSGroupLayer(name=self.conf.get('name'), title=self.conf.get('title'),
                                   this=this_layer, layers=layers)
         return layer
 
 class LayerConfiguration(ConfigurationBase):
-    optional_keys = set('min_res max_res min_scale max_scale sources layers'.split())
+    optional_keys = set('min_res max_res min_scale max_scale legendurl sources layers'.split())
     required_keys = set('name title'.split())
+
     @memoize
     def wms_layer(self):
         sources = []
         fi_sources = []
         lg_sources = []
-        for source_name in self.conf['sources']:
+        
+        lg_sources_configured = False
+        if self.conf.get('legendurl'):
+            legend_url = self.conf['legendurl']
+            lg_sources.append(WMSSourceConfiguration.static_legend_source(legend_url, self.context))
+            lg_sources_configured = True
+        
+        for source_name in self.conf.get('sources', []):
             fi_source_names = []
             lg_source_names = []
             if source_name in self.context.caches:
@@ -717,23 +783,25 @@ class LayerConfiguration(ConfigurationBase):
                 lg_source_names = [source_name]
             else:
                 raise ConfigurationError('source/cache "%s" not found' % source_name)
-            sources.append(map_layer)
+            
+            if map_layer:
+                sources.append(map_layer)
             
             for fi_source_name in fi_source_names:
                 if not hasattr(self.context.sources[fi_source_name], 'fi_source'): continue
                 fi_source = self.context.sources[fi_source_name].fi_source()
                 if fi_source:
                     fi_sources.append(fi_source)
-
-            for lg_source_name in lg_source_names:
-                if not hasattr(self.context.sources[lg_source_name], 'lg_source'): continue
-                lg_source = self.context.sources[lg_source_name].lg_source()
-                if lg_source:
-                    lg_sources.append(lg_source)
-            
+            if not lg_sources_configured:
+                for lg_source_name in lg_source_names:
+                    if not hasattr(self.context.sources[lg_source_name], 'lg_source'): continue
+                    lg_source = self.context.sources[lg_source_name].lg_source()
+                    if lg_source:
+                        lg_sources.append(lg_source)
+                
         res_range = resolution_range(self.conf)
         
-        layer = WMSLayer({'title': self.conf['title'], 'name': self.conf['name']},
+        layer = WMSLayer(self.conf.get('name'), self.conf.get('title'),
                          sources, fi_sources, lg_sources, res_range=res_range)
         return layer
     
@@ -752,11 +820,23 @@ class LayerConfiguration(ConfigurationBase):
                 md['name_internal'] = md['name_path'][0] + '_' + md['name_path'][1]
                 md['format'] = self.context.caches[cache_name].conf['format']
             
-                tile_layers.append(TileLayer(md, cache_source))
+                tile_layers.append(TileLayer(self.conf['name'], self.conf['title'],
+                                             md, cache_source))
         
         return tile_layers
         
 
+def fi_xslt_transformers(conf, context):
+    fi_transformers = {}
+    fi_xslt = conf.get('featureinfo_xslt')
+    if fi_xslt:
+        if not has_xslt_support:
+            raise ValueError('featureinfo_xslt requires lxml. Please install.')
+        for info_type, fi_xslt in fi_xslt.items():
+            fi_xslt = context.globals.abspath(fi_xslt)
+            fi_transformers[info_type] = XSLTransformer(fi_xslt)
+    return fi_transformers
+
 class ServiceConfiguration(ConfigurationBase):
     optional_keys = set('wms tms kml demo'.split())
     
@@ -794,15 +874,28 @@ class ServiceConfiguration(ConfigurationBase):
         tile_layers = self.tile_layers(conf)
         attribution = conf.get('attribution')
         strict = self.context.globals.get_value('strict', conf, global_key='wms.strict')
-        layers = odict()
+        on_source_errors = self.context.globals.get_value('on_source_errors',
+            conf, global_key='wms.on_source_errors')
         root_layer = self.context.wms_root_layer.wms_layer()
-        if not root_layer.md.get('title'):
-            root_layer.md['title'] = md.get('title')
-        image_formats = self.context.globals.get_value('image_formats', conf, global_key='wms.image_formats')
+        if not root_layer.title:
+            # set title of root layer to WMS title
+            root_layer.title = md.get('title')
+        concurrent_layer_renderer = self.context.globals.get_value(
+            'concurrent_layer_renderer', conf,
+            global_key='wms.concurrent_layer_renderer')
+        image_formats = self.context.globals.get_value('image_formats', conf,
+                                                       global_key='wms.image_formats')
+        info_types = conf.get('featureinfo_types')
         srs = self.context.globals.get_value('srs', conf, global_key='wms.srs')
         self.context.globals.base_config.wms.srs = srs
-        return WMSServer(root_layer, md, attribution=attribution, image_formats=image_formats,
-            srs=srs, tile_layers=tile_layers, strict=strict)
+        server = WMSServer(root_layer, md, attribution=attribution,
+            image_formats=image_formats, info_types=info_types,
+            srs=srs, tile_layers=tile_layers, strict=strict, on_error=on_source_errors,
+            concurrent_layer_renderer=concurrent_layer_renderer)
+        
+        server.fi_transformers = fi_xslt_transformers(conf, self.context)
+        
+        return server
 
     def demo_service(self, conf):
         md = self.context.services.conf.get('wms', {}).get('md', {}).copy()
@@ -824,7 +917,7 @@ def load_yaml_file(file):
     else:
         return yaml.load(file)
 
-def load_configuration(mapproxy_conf):
+def load_configuration(mapproxy_conf, seed=False):
     if hasattr(mapproxy_conf, 'read'):
         conf_data = mapproxy_conf.read()
         conf_base_dir = os.getcwd()
@@ -840,7 +933,7 @@ def load_configuration(mapproxy_conf):
             log.warn('found `base` option in base config but recursive inheritance is not supported.')
         conf_dict = merge_dict(conf_dict, base_dict)
     
-    return ProxyConfiguration(conf_dict, conf_base_dir=conf_base_dir)
+    return ProxyConfiguration(conf_dict, conf_base_dir=conf_base_dir, seed=seed)
 
 def merge_dict(conf, base):
     """
@@ -854,4 +947,29 @@ def merge_dict(conf, base):
                 merge_dict(v, base[k])
             else:
                 base[k] = v
-    return base
\ No newline at end of file
+    return base
+
+def parse_color(color):
+    """
+    >>> parse_color((100, 12, 55))
+    (100, 12, 55)
+    >>> parse_color('0xff0530')
+    (255, 5, 48)
+    >>> parse_color('#FF0530')
+    (255, 5, 48)
+    """
+    if isinstance(color, (list, tuple)):
+        return color
+    if not isinstance(color, basestring):
+        raise ValueError('color needs to be a tuple/list or 0xrrggbb/#rrggbb string')
+    
+    if color.startswith('0x'):
+        color = color[2:]
+    if color.startswith('#'):
+        color = color[1:]
+    
+    r, g, b = map(lambda x: int(x, 16), [color[:2], color[2:4], color[4:6]])
+    
+    return r, g, b
+    
+    
\ No newline at end of file
