@@ -39,6 +39,13 @@ class MapBBOXError(Exception):
 
 class MapLayer(object):
     res_range = None
+    opacity = None
+    transparent = False
+    
+    def is_opaque(self):
+        if self.opacity is None:
+            return not self.transparent
+        return self.opacity >= 0.99
     
     def check_res_range(self, query):
         if (self.res_range and
@@ -47,6 +54,9 @@ class MapLayer(object):
     
     def get_map(self, query):
         raise NotImplementedError
+    
+    def combined_layer(self, other, query):
+        return None
 
 class InfoLayer(object):
     def get_info(self, query):
@@ -142,6 +152,8 @@ class DefaultMapExtent(MapExtent):
         MapExtent.__init__(self, (-180, -90, 180, 90), SRS(4326))
 
 def merge_layer_extents(layers):
+    if not layers:
+        return DefaultMapExtent()
     layers = layers[:]
     extent = layers.pop().extent
     for layer in layers:
@@ -149,15 +161,16 @@ def merge_layer_extents(layers):
     return extent
 
 class ResolutionConditional(MapLayer):
-    def __init__(self, one, two, resolution, srs, extent):
+    def __init__(self, one, two, resolution, srs, extent, opacity=None):
         self.one = one
         self.two = two
-        self.res_range = _calculate_res_range([one, two])
+        self.res_range = merge_layer_res_ranges([one, two])
         self.resolution = resolution
         self.srs = srs
         
         #TODO
         self.transparent = self.one.transparent
+        self.opacity = opacity
         self.extent = extent
     
     def get_map(self, query):
@@ -180,16 +193,17 @@ class SRSConditional(MapLayer):
     PROJECTED = 'PROJECTED'
     GEOGRAPHIC = 'GEOGRAPHIC'
     
-    def __init__(self, layers, extent, transparent=False):
+    def __init__(self, layers, extent, transparent=False, opacity=None):
         self.transparent = transparent
         # TODO geographic/projected fallback
         self.srs_map = {}
-        self.res_range = _calculate_res_range([l[0] for l in layers])
+        self.res_range = merge_layer_res_ranges([l[0] for l in layers])
         for layer, srss in layers:
             for srs in srss:
                 self.srs_map[srs] = layer
         
         self.extent = extent
+        self.opacity = opacity
     
     def get_map(self, query):
         self.check_res_range(query)
@@ -227,7 +241,7 @@ class DirectMapLayer(MapLayer):
         return self.source.get_map(query)
 
 
-def _calculate_res_range(layers):
+def merge_layer_res_ranges(layers):
     ranges = [s.res_range for s in layers
               if hasattr(s, 'res_range')]
     if ranges:
@@ -237,12 +251,13 @@ def _calculate_res_range(layers):
 
 
 class CacheMapLayer(MapLayer):
-    def __init__(self, tile_manager, extent=None, resampling=None):
+    def __init__(self, tile_manager, extent=None, resampling=None, opacity=None):
         self.tile_manager = tile_manager
         self.grid = tile_manager.grid
+        self.opacity = opacity
         self.resampling = resampling or base_config().image.resampling_method
         self.extent = extent or map_extent_from_grid(self.grid)
-        self.res_range = _calculate_res_range(self.tile_manager.sources)
+        self.res_range = merge_layer_res_ranges(self.tile_manager.sources)
         self.transparent = tile_manager.transparent
     
     def get_map(self, query):
