@@ -16,10 +16,13 @@
 
 import platform
 
+__all__ = ['Proj', 'transform', 'set_datapath', 'ProjInitError']
+
 if platform.system() == "Java":
     from mapproxy.platform.jython.proj import (
         Proj, transform, set_datapath, ProjInitError
     )
+    Proj, transform, set_datapath, ProjInitError # prevent pyflake warnings
 else:
     from mapproxy.platform.cpython.proj import (
         Proj, transform, set_datapath, ProjInitError
