@@ -17,8 +17,10 @@
 """
 Service requests (parsing, handling, etc).
 """
+from __future__ import with_statement
+import codecs
 from mapproxy.request.wms import exception
-from mapproxy.config import base_config
+from mapproxy.config import base_config, abspath
 from mapproxy.exception import RequestError
 from mapproxy.srs import SRS, make_lin_transf
 from mapproxy.request.base import RequestParams, BaseRequest, split_mime_type
@@ -35,12 +37,18 @@ class WMSMapRequestParams(RequestParams):
     and height, ``layers`` returns an iterator of all layers, etc. 
     
     """
-    @property
-    def layers(self):
+    def _get_layers(self):
         """
         List with all layer names.
         """
         return sum((layers.split(',') for layers in self.params.get_all('layers')), [])
+    def _set_layers(self, layers):
+        if isinstance(layers, (list, tuple)):
+            layers = ','.join(layers)
+        self.params['layers'] = layers
+    layers = property(_get_layers, _set_layers)
+    del _get_layers
+    del _set_layers
     
     def _get_bbox(self):
         """
@@ -240,9 +248,28 @@ class WMSMapRequest(WMSRequest):
     def copy(self):
         return self.__class__(param=self.params.copy(), url=self.url)
     
+
+class Version(object):
+    _versions = {}
+    def __new__(cls, version):
+        if version in cls._versions:
+            return cls._versions[version]
+        version_obj = object.__new__(cls)
+        version_obj.__init__(version)
+        cls._versions[version] = version_obj
+        return version_obj
+    def __init__(self, version):
+        self.parts = tuple(int(x) for x in version.split('.'))
     
+    def __cmp__(self, other):
+        if not isinstance(other, Version):
+            return NotImplemented
+        return cmp(self.parts, other.parts)
+    def __repr__(self):
+        return "Version('%s')" % ('.'.join(str(part) for part in self.parts),)
 
 class WMS100MapRequest(WMSMapRequest):
+    version = Version('1.0.0')
     xml_exception_handler = exception.WMS100ExceptionHandler
     fixed_params = {'request': 'map', 'wmtver': '1.0.0'}
     expected_param = ['wmtver', 'request', 'layers', 'styles', 'srs', 'bbox', 
@@ -259,6 +286,7 @@ class WMS100MapRequest(WMSMapRequest):
         return params
 
 class WMS110MapRequest(WMSMapRequest):
+    version = Version('1.1.0')
     fixed_params = {'request': 'GetMap', 'version': '1.1.0', 'service': 'WMS'}
     xml_exception_handler = exception.WMS110ExceptionHandler
     
@@ -266,6 +294,7 @@ class WMS110MapRequest(WMSMapRequest):
         del self.params['wmtver']
 
 class WMS111MapRequest(WMSMapRequest):
+    version = Version('1.1.1')
     fixed_params = {'request': 'GetMap', 'version': '1.1.1', 'service': 'WMS'}
     xml_exception_handler = exception.WMS111ExceptionHandler
     
@@ -290,6 +319,7 @@ class WMS130MapRequestParams(WMSMapRequestParams):
 
 
 class WMS130MapRequest(WMSMapRequest):
+    version = Version('1.3.0')
     request_params = WMS130MapRequestParams
     xml_exception_handler = exception.WMS130ExceptionHandler
     fixed_params = {'request': 'GetMap', 'version': '1.3.0', 'service': 'WMS'}
@@ -407,11 +437,13 @@ class WMSLegendGraphicRequest(WMSMapRequest):
     
 
 class WMS111LegendGraphicRequest(WMSLegendGraphicRequest):
+    version = Version('1.1.1')
     fixed_params = WMSLegendGraphicRequest.fixed_params.copy()
     fixed_params['version'] = '1.1.1'
     xml_exception_handler = exception.WMS111ExceptionHandler
     
 class WMS130LegendGraphicRequest(WMSLegendGraphicRequest):
+    version = Version('1.3.0')
     fixed_params = WMSLegendGraphicRequest.fixed_params.copy()
     fixed_params['version'] = '1.3.0'
     xml_exception_handler = exception.WMS130ExceptionHandler
@@ -424,6 +456,7 @@ class WMSFeatureInfoRequest(WMSMapRequest):
         WMSMapRequest.validate_format(self)
 
 class WMS111FeatureInfoRequest(WMSFeatureInfoRequest):
+    version = Version('1.1.1')
     request_params = WMSFeatureInfoRequestParams
     xml_exception_handler = exception.WMS111ExceptionHandler
     request_handler_name = 'featureinfo'
@@ -432,6 +465,7 @@ class WMS111FeatureInfoRequest(WMSFeatureInfoRequest):
     expected_param = WMSMapRequest.expected_param[:] + ['query_layers', 'x', 'y']
 
 class WMS110FeatureInfoRequest(WMSFeatureInfoRequest):
+    version = Version('1.1.0')
     request_params = WMSFeatureInfoRequestParams
     xml_exception_handler = exception.WMS110ExceptionHandler
     request_handler_name = 'featureinfo'
@@ -440,6 +474,7 @@ class WMS110FeatureInfoRequest(WMSFeatureInfoRequest):
     expected_param = WMSMapRequest.expected_param[:] + ['query_layers', 'x', 'y']
 
 class WMS100FeatureInfoRequest(WMSFeatureInfoRequest):
+    version = Version('1.0.0')
     request_params = WMSFeatureInfoRequestParams
     xml_exception_handler = exception.WMS100ExceptionHandler
     request_handler_name = 'featureinfo'
@@ -456,6 +491,9 @@ class WMS100FeatureInfoRequest(WMSFeatureInfoRequest):
         return params
 
 class WMS130FeatureInfoRequest(WMS130MapRequest):
+    # XXX: this class inherits from WMS130MapRequest to reuse
+    # the axis order stuff
+    version = Version('1.3.0')
     request_params = WMS130FeatureInfoRequestParams
     xml_exception_handler = exception.WMS130ExceptionHandler
     request_handler_name = 'featureinfo'
@@ -482,10 +520,10 @@ class WMS130FeatureInfoRequest(WMS130MapRequest):
         del params['x']
         del params['y']
         return params
-    
+
     def validate_format(self):
-        if self.non_strict: return
-        WMS130MapRequest.validate_format(self)
+        if self.non_strict: return 
+        WMSMapRequest.validate_format(self)
 
 class WMSCapabilitiesRequest(WMSRequest):
     request_handler_name = 'capabilities'
@@ -503,6 +541,7 @@ class WMSCapabilitiesRequest(WMSRequest):
     
 
 class WMS100CapabilitiesRequest(WMSCapabilitiesRequest):
+    version = Version('1.0.0')
     capabilities_template = 'wms100capabilities.xml'
     fixed_params = {'request': 'capabilities', 'wmtver': '1.0.0'}
     
@@ -512,6 +551,7 @@ class WMS100CapabilitiesRequest(WMSCapabilitiesRequest):
     
 
 class WMS110CapabilitiesRequest(WMSCapabilitiesRequest):
+    version = Version('1.1.0')
     capabilities_template = 'wms110capabilities.xml'
     mime_type = 'application/vnd.ogc.wms_xml'
     fixed_params = {'request': 'GetCapabilities', 'version': '1.1.0', 'service': 'WMS'}
@@ -521,6 +561,7 @@ class WMS110CapabilitiesRequest(WMSCapabilitiesRequest):
         return exception.WMS110ExceptionHandler()
 
 class WMS111CapabilitiesRequest(WMSCapabilitiesRequest):
+    version = Version('1.1.1')
     capabilities_template = 'wms111capabilities.xml'
     mime_type = 'application/vnd.ogc.wms_xml'
     fixed_params = {'request': 'GetCapabilities', 'version': '1.1.1', 'service': 'WMS'}
@@ -531,6 +572,7 @@ class WMS111CapabilitiesRequest(WMSCapabilitiesRequest):
     
 
 class WMS130CapabilitiesRequest(WMSCapabilitiesRequest):
+    version = Version('1.3.0')
     capabilities_template = 'wms130capabilities.xml'
     fixed_params = {'request': 'GetCapabilities', 'version': '1.3.0', 'service': 'WMS'}
     
@@ -538,25 +580,6 @@ class WMS130CapabilitiesRequest(WMSCapabilitiesRequest):
     def exception_handler(self):
         return exception.WMS130ExceptionHandler()
     
-class Version(object):
-    _versions = {}
-    def __new__(cls, version):
-        if version in cls._versions:
-            return cls._versions[version]
-        version_obj = object.__new__(cls)
-        version_obj.__init__(version)
-        cls._versions[version] = version_obj
-        return version_obj
-    def __init__(self, version):
-        self.parts = tuple(int(x) for x in version.split('.'))
-    
-    def __cmp__(self, other):
-        if not isinstance(other, Version):
-            return NotImplemented
-        return cmp(self.parts, other.parts)
-    def __repr__(self):
-        return "Version('%s')" % ('.'.join(str(part) for part in self.parts),)
-
 request_mapping = {Version('1.0.0'): {'featureinfo': WMS100FeatureInfoRequest,
                                       'map': WMS100MapRequest,
                                       'capabilities': WMS100CapabilitiesRequest},
@@ -654,8 +677,49 @@ def create_request(req_data, param, req_type='map', version='1.1.1'):
     elif 'format' in param:
         req_data['format'] = param['format']
     
+    if 'info_format' in param:
+        req_data['info_format'] = param['info_format']
+    
     if 'transparent' in req_data:
         # we don't want a boolean
         req_data['transparent'] = str(req_data['transparent'])
     
-    return request_mapping[Version(version)][req_type](url=url, param=req_data)
\ No newline at end of file
+    if req_data.get('sld', '').startswith('file://'):
+        sld_path = req_data['sld'][len('file://'):]
+        sld_path = abspath(sld_path)
+        with codecs.open(sld_path, 'UTF-8') as f:
+            req_data['sld_body'] = f.read()
+        del req_data['sld']
+    
+    return request_mapping[Version(version)][req_type](url=url, param=req_data)
+
+
+info_formats = {
+    Version('1.3.0'): (('text', 'text/plain'),
+                       ('html', 'text/html'),
+                       ('xml', 'text/xml'),
+                      ),
+    None: (('text', 'text/plain'),
+           ('html', 'text/html'),
+           ('xml', 'application/vnd.ogc.gml'),
+          )
+}
+
+
+def infotype_from_mimetype(version, mime_type):
+    if version in info_formats:
+        formats = info_formats[version]
+    else:
+        formats = info_formats[None] # default
+    for t, m in formats:
+        if m == mime_type: return t
+
+def mimetype_from_infotype(version, info_type):
+    if version in info_formats:
+        formats = info_formats[version]
+    else:
+        formats = info_formats[None] # default
+    for t, m in formats:
+        if t == info_type: return m
+    return 'text/plain'
+    
