@@ -31,6 +31,7 @@ from mapproxy.source import SourceError
 from mapproxy.srs import SRS
 from mapproxy.grid import default_bboxs
 from mapproxy.image import BlankImageSource
+from mapproxy.util.ext.odict import odict
 
 import logging
 log = logging.getLogger(__name__)
@@ -87,8 +88,34 @@ class TileServer(Server):
         internal_layer = self._internal_layer(tile_request.layer)
         if internal_layer is None:
             raise RequestError('unknown layer: ' + tile_request.layer, request=tile_request)
+        self.authorize_tile_layer(internal_layer.name, tile_request.http.environ)
         return internal_layer
     
+    def authorize_tile_layer(self, layer_name, env):
+        if 'mapproxy.authorize' in env:
+            result = env['mapproxy.authorize']('tms', [layer_name])
+            if result['authorized'] == 'full':
+                return
+            if result['authorized'] == 'partial':
+                if result['layers'].get(layer_name, {}).get('tile', False) == True:
+                    return
+            raise RequestError('forbidden', status=403)
+    
+    def authorized_tile_layers(self, env):
+        if 'mapproxy.authorize' in env:
+            result = env['mapproxy.authorize']('tms', [l for l in self.layers])
+            if result['authorized'] == 'full':
+                return self.layers
+            if result['authorized'] == 'none':
+                raise RequestError('forbidden', status=403)
+            allowed_layers = odict()
+            for layer in self.layers.itervalues():
+                if result['layers'].get(layer.name, {}).get('tile', False) == True:
+                    allowed_layers[layer.name] = layer
+            return allowed_layers
+        else:
+            return self.layers
+    
     def tms_capabilities(self, tms_request):
         """
         :return: the rendered tms capabilities
@@ -97,9 +124,11 @@ class TileServer(Server):
         service = self._service_md(tms_request)
         if hasattr(tms_request, 'layer'):
             layer = self.layer(tms_request)
+            self.authorize_tile_layer(layer.name, tms_request.http.environ)
             result = self._render_layer_template(layer, service)
         else:
-            result = self._render_template(service)
+            layers = self.authorized_tile_layers(tms_request.http.environ)
+            result = self._render_template(layers, service)
 
         return Response(result, mimetype='text/xml')
     
@@ -108,20 +137,22 @@ class TileServer(Server):
         md['url'] = map_request.http.base_url
         return md
     
-    def _render_template(self, service):
+    def _render_template(self, layers, service):
         template = get_template(self.template_file)
-        return template.substitute(service=bunch(default='', **service), layers=self.layers)
+        return template.substitute(service=bunch(default='', **service), layers=layers)
     
     def _render_layer_template(self, layer, service):
         template = get_template(self.layer_template_file)
         return template.substitute(service=bunch(default='', **service), layer=layer)
 
 class TileLayer(object):
-    def __init__(self, md, tile_manager):
+    def __init__(self, name, title, md, tile_manager):
         """
         :param md: the layer metadata
         :param tile_manager: the layer tile manager
         """
+        self.name = name
+        self.title = title
         self.md = md
         self.tile_manager = tile_manager
         self.grid = TileServiceGrid(tile_manager.grid)
@@ -296,3 +327,19 @@ class TileServiceGrid(object):
             z *= 2
         return self.grid.limit_tile((x, y, z))
     
+    def external_tile_coord(self, tile_coord, use_profiles):
+        """
+        Converts internal tile coords to external tile coords.
+        
+        :param tile_coord: the internal tile coord
+        :param use_profiles: True if the tile service supports global 
+                             profiles (see `mapproxy.core.server.TileServer`)
+        """
+        x, y, z = tile_coord
+        if z < 0:
+            return None
+        if use_profiles and self._skip_first_level:
+            z -= 1
+        if self._skip_odd_level:
+            z //= 2
+        return (x, y, z)
