@@ -20,10 +20,9 @@ Retrieve maps/information from WMS servers.
 import sys
 from mapproxy.request.base import split_mime_type
 from mapproxy.cache.legend import Legend, legend_identifier
-from mapproxy.image import concat_legends
+from mapproxy.image import concat_legends, make_transparent
 from mapproxy.layer import MapExtent, DefaultMapExtent, BlankImage, LegendQuery
 from mapproxy.source import Source, InfoSource, SourceError, LegendSource
-from mapproxy.srs import SRS
 from mapproxy.client.http import HTTPClientError
 from mapproxy.util import reraise_exception
 
@@ -32,10 +31,14 @@ log = logging.getLogger(__name__)
 
 class WMSSource(Source):
     supports_meta_tiles = True
-    def __init__(self, client, transparent=False, coverage=None, res_range=None):
+    def __init__(self, client, transparent=False, coverage=None, res_range=None, opacity=None,
+                 transparent_color=None, transparent_color_tolerance=None):
         Source.__init__(self)
         self.client = client
-        self.transparent = transparent
+        self.transparent_color = transparent_color
+        self.transparent_color_tolerance = transparent_color_tolerance
+        self.transparent = True if transparent_color else transparent
+        self.opacity = opacity
         self.coverage = coverage
         self.res_range = res_range
         if self.coverage:
@@ -50,24 +53,45 @@ class WMSSource(Source):
         if self.coverage and not self.coverage.intersects(query.bbox, query.srs):
             raise BlankImage()
         try:
-            return self.client.get_map(query)
+            resp = self.client.get_map(query)
+            if self.transparent_color:
+                resp = make_transparent(resp, self.transparent_color,
+                                        self.transparent_color_tolerance)
+            resp.opacity = self.opacity
+            return resp
+            
         except HTTPClientError, e:
             reraise_exception(SourceError(e.args[0]), sys.exc_info())
+    
+    def combined_layer(self, other, query):
+        if not isinstance(other, WMSSource):
+            return None
+        
+        if self.opacity is not None or other.opacity is not None:
+            return None
+        
+        client = self.client.combined_client(other.client, query)
+        if not client:
+            return None
+        
+        return WMSSource(client, transparent=self.transparent)
         
-
 class WMSInfoSource(InfoSource):
-    def __init__(self, client):
+    def __init__(self, client, fi_transformer=None):
         self.client = client
+        self.fi_transformer = fi_transformer
     
     def get_info(self, query):
-        return self.client.get_info(query).read()
-        
+        doc = self.client.get_info(query)
+        if self.fi_transformer:
+            doc = self.fi_transformer(doc)
+        return doc
+    
+
 class WMSLegendSource(LegendSource):
     def __init__(self, clients, legend_cache):
         self.clients = clients
-        self.identifier = legend_identifier(
-            [(c.request_template.url, c.request_template.params.layer)
-             for c in self.clients])
+        self.identifier = legend_identifier([c.identifier for c in self.clients])
         self._cache = legend_cache
         self._size = None
     
