@@ -181,4 +181,12 @@ class XPathValidator(object):
             else:
                 eq_(self.xml.xpath(xpath)[0], expected)
     def xpath(self, xpath):
-        return self.xml.xpath(xpath)
\ No newline at end of file
+        return self.xml.xpath(xpath)
+
+
+def strip_whitespace(text):
+    """
+    >>> strip_whitespace(' <foo> bar\\n zing\\t1')
+    '<foo>barzing1'
+    """
+    return re.sub('\s+', '', text)
\ No newline at end of file
