@@ -19,21 +19,25 @@ import os
 import time
 import shutil
 import tempfile
-from mapproxy.seed import seed_from_yaml_conf
+from mapproxy.config.loader import load_configuration
+from mapproxy.seed.seeder import seed
+from mapproxy.seed.cleanup import cleanup
+from mapproxy.seed.config import load_seed_tasks_conf
 
 from mapproxy.test.http import mock_httpd
 from mapproxy.test.image import tmp_image
 
 FIXTURE_DIR = os.path.join(os.path.dirname(__file__), 'fixture')
 
-class TestSeed(object):
+class SeedTestBase(object):
     def setup(self):
         self.dir = tempfile.mkdtemp()
-        shutil.copy(os.path.join(FIXTURE_DIR, 'seed_mapproxy.yaml'), self.dir)
-        shutil.copy(os.path.join(FIXTURE_DIR, 'seed.yaml'), self.dir)
-        self.seed_conf_file = os.path.join(self.dir, 'seed.yaml')
-        self.mapproxy_conf_file = os.path.join(self.dir, 'seed_mapproxy.yaml')
-        
+        shutil.copy(os.path.join(FIXTURE_DIR, self.seed_conf_name), self.dir)
+        shutil.copy(os.path.join(FIXTURE_DIR, self.mapproxy_conf_name), self.dir)
+        self.seed_conf_file = os.path.join(self.dir, self.seed_conf_name)
+        self.mapproxy_conf_file = os.path.join(self.dir, self.mapproxy_conf_name)
+        self.mapproxy_conf = load_configuration(self.mapproxy_conf_file, seed=True)
+    
     def teardown(self):
         shutil.rmtree(self.dir)
     
@@ -50,8 +54,17 @@ class TestSeed(object):
             os.utime(tile, (timestamp, timestamp))
         return tile
     
+    def tile_exists(self, coord):
+        tile_dir = os.path.join(self.dir, 'cache/one_EPSG4326/%02d/000/000/%03d/000/000/' %
+                                (coord[2], coord[0]))
+        tile = os.path.join(tile_dir + '%03d.png' % coord[1])
+        return os.path.exists(tile)
+
     def test_seed_dry_run(self):
-       seed_from_yaml_conf(self.seed_conf_file, self.mapproxy_conf_file, verbose=False, dry_run=True)
+        seed_conf  = load_seed_tasks_conf(self.seed_conf_file, self.mapproxy_conf)
+        tasks, cleanup_tasks = seed_conf.seeds(), seed_conf.cleanups()
+        seed(tasks, verbose=False, dry_run=True)
+        cleanup(cleanup_tasks, verbose=False, dry_run=True)
     
     def test_seed(self):
         with tmp_image((256, 256), format='png') as img:
@@ -61,13 +74,23 @@ class TestSeed(object):
                                   '&width=256&height=128&srs=EPSG:4326'},
                             {'body': img_data, 'headers': {'content-type': 'image/png'}})
             with mock_httpd(('localhost', 42423), [expected_req]):
-                seed_from_yaml_conf(self.seed_conf_file, self.mapproxy_conf_file, verbose=True, dry_run=False)
+                seed_conf  = load_seed_tasks_conf(self.seed_conf_file, self.mapproxy_conf)
+                tasks, cleanup_tasks = seed_conf.seeds(), seed_conf.cleanups()
+                seed(tasks, verbose=False, dry_run=False)
+                cleanup(cleanup_tasks, verbose=False, dry_run=False)
 
     def test_reseed_uptodate(self):
         # tile already there.
         self.make_tile((0, 0, 0))
-        seed_from_yaml_conf(self.seed_conf_file, self.mapproxy_conf_file, verbose=False)
-    
+        seed_conf  = load_seed_tasks_conf(self.seed_conf_file, self.mapproxy_conf)
+        tasks, cleanup_tasks = seed_conf.seeds(), seed_conf.cleanups()
+        seed(tasks, verbose=False, dry_run=False)
+        cleanup(cleanup_tasks, verbose=False, dry_run=False)
+
+class TestSeedOldConfiguration(SeedTestBase):
+    seed_conf_name = 'seed_old.yaml'
+    mapproxy_conf_name = 'seed_mapproxy.yaml'
+
     def test_reseed_remove_before(self):
         # tile already there but too old
         t000 = self.make_tile((0, 0, 0), timestamp=time.time() - (60*60*25))
@@ -82,8 +105,49 @@ class TestSeed(object):
                                   '&width=256&height=128&srs=EPSG:4326'},
                             {'body': img_data, 'headers': {'content-type': 'image/png'}})
             with mock_httpd(('localhost', 42423), [expected_req]):
-                seed_from_yaml_conf(self.seed_conf_file, self.mapproxy_conf_file, verbose=True, dry_run=False)
+                seed_conf  = load_seed_tasks_conf(self.seed_conf_file, self.mapproxy_conf)
+                tasks, cleanup_tasks = seed_conf.seeds(), seed_conf.cleanups()
+                seed(tasks, verbose=True, dry_run=False)
+                cleanup(cleanup_tasks, verbose=False, dry_run=False)
         
         assert os.path.exists(t000)
         assert os.path.getmtime(t000) - 5 < time.time() < os.path.getmtime(t000) + 5
-        assert not os.path.exists(t001)
\ No newline at end of file
+        assert not os.path.exists(t001)
+
+class TestSeed(SeedTestBase):
+    seed_conf_name = 'seed.yaml'
+    mapproxy_conf_name = 'seed_mapproxy.yaml'
+    
+    def test_cleanup_levels(self):
+        seed_conf  = load_seed_tasks_conf(self.seed_conf_file, self.mapproxy_conf)
+        cleanup_tasks = seed_conf.cleanups(['cleanup'])
+        
+        self.make_tile((0, 0, 0))
+        self.make_tile((0, 0, 1))
+        self.make_tile((0, 0, 2))
+        self.make_tile((0, 0, 3))
+        
+        cleanup(cleanup_tasks, verbose=False, dry_run=False)
+        assert not self.tile_exists((0, 0, 0))
+        assert not self.tile_exists((0, 0, 1))
+        assert self.tile_exists((0, 0, 2))
+        assert not self.tile_exists((0, 0, 3))
+
+    def test_cleanup_coverage(self):
+        seed_conf = load_seed_tasks_conf(self.seed_conf_file, self.mapproxy_conf)
+        cleanup_tasks = seed_conf.cleanups(['with_coverage'])
+        
+        self.make_tile((0, 0, 0))
+        self.make_tile((1, 0, 1))
+        self.make_tile((2, 0, 2))
+        self.make_tile((2, 0, 3))
+        self.make_tile((4, 0, 3))
+        
+        cleanup(cleanup_tasks, verbose=False, dry_run=False)
+        assert not self.tile_exists((0, 0, 0))
+        assert not self.tile_exists((1, 0, 1))
+        assert self.tile_exists((2, 0, 2))
+        assert not self.tile_exists((2, 0, 3))
+        assert self.tile_exists((4, 0, 3))
+
+
