@@ -3,16 +3,14 @@ import yaml
 from cStringIO import StringIO
 from mapproxy.srs import SRS
 from mapproxy.config.loader import (
-    GridConfiguration,
     ProxyConfiguration,
-    WMSSourceConfiguration,
     load_services,
     merge_dict,
 )
 from mapproxy.cache.tile import TileManager
 
 from mapproxy.test.unit.test_grid import assert_almost_equal_bbox
-from nose.tools import eq_, assert_almost_equal
+from nose.tools import eq_
 from nose.plugins.skip import SkipTest
 
 class TestLayerConfiguration(object):
@@ -38,17 +36,17 @@ class TestLayerConfiguration(object):
         root = conf.wms_root_layer.wms_layer()
         
         # no root layer defined
-        eq_(root.md['title'], None)
-        eq_(root.md['name'], None)
+        eq_(root.title, None)
+        eq_(root.name, None)
         layers = root.child_layers()
         
         # names are in order
         eq_(layers.keys(), ['one', 'two', 'three'])
         
         eq_(len(layers), 3)
-        eq_(layers['one'].md['title'], 'Layer One')
-        eq_(layers['two'].md['title'], 'Layer Two')
-        eq_(layers['three'].md['title'], 'Layer Three')
+        eq_(layers['one'].title, 'Layer One')
+        eq_(layers['two'].title, 'Layer Two')
+        eq_(layers['three'].title, 'Layer Three')
         
         layers_conf = conf.layers
         eq_(len(layers_conf), 3)
@@ -70,17 +68,17 @@ class TestLayerConfiguration(object):
         root = conf.wms_root_layer.wms_layer()
         
         # no root layer defined
-        eq_(root.md['title'], None)
-        eq_(root.md['name'], None)
+        eq_(root.title, None)
+        eq_(root.name, None)
         layers = root.child_layers()
         
         # names might not be in order
         # layers.keys() != ['one', 'two', 'three']
         
         eq_(len(layers), 3)
-        eq_(layers['one'].md['title'], 'Layer One')
-        eq_(layers['two'].md['title'], 'Layer Two')
-        eq_(layers['three'].md['title'], 'Layer Three')
+        eq_(layers['one'].title, 'Layer One')
+        eq_(layers['two'].title, 'Layer Two')
+        eq_(layers['three'].title, 'Layer Three')
 
     def test_with_root(self):
         conf = self._test_conf('''
@@ -98,17 +96,17 @@ class TestLayerConfiguration(object):
         conf = ProxyConfiguration(conf)
         root = conf.wms_root_layer.wms_layer()
         
-        eq_(root.md['title'], 'Root Layer')
-        eq_(root.md['name'], 'root')
+        eq_(root.title, 'Root Layer')
+        eq_(root.name, 'root')
         layers = root.child_layers()
         
         # names are in order
         eq_(layers.keys(), ['root', 'one', 'two'])
         
         eq_(len(layers), 3)
-        eq_(layers['root'].md['title'], 'Root Layer')
-        eq_(layers['one'].md['title'], 'Layer One')
-        eq_(layers['two'].md['title'], 'Layer Two')
+        eq_(layers['root'].title, 'Root Layer')
+        eq_(layers['one'].title, 'Layer One')
+        eq_(layers['two'].title, 'Layer Two')
         
         layers_conf = conf.layers
         eq_(len(layers_conf), 2)
@@ -128,8 +126,8 @@ class TestLayerConfiguration(object):
         conf = ProxyConfiguration(conf)
         root = conf.wms_root_layer.wms_layer()
         
-        eq_(root.md['title'], 'Root Layer')
-        eq_(root.md['name'], None)
+        eq_(root.title, 'Root Layer')
+        eq_(root.name, None)
 
         layers = root.child_layers()
         # names are in order
@@ -148,8 +146,8 @@ class TestLayerConfiguration(object):
         conf = ProxyConfiguration(conf)
         root = conf.wms_root_layer.wms_layer()
         
-        eq_(root.md['title'], None)
-        eq_(root.md['name'], None)
+        eq_(root.title, None)
+        eq_(root.name, None)
 
         layers = root.child_layers()
         # names are in order
@@ -182,8 +180,8 @@ class TestLayerConfiguration(object):
         conf = ProxyConfiguration(conf)
         root = conf.wms_root_layer.wms_layer()
         
-        eq_(root.md['title'], 'Root Layer')
-        eq_(root.md['name'], None)
+        eq_(root.title, 'Root Layer')
+        eq_(root.name, None)
 
         layers = root.child_layers()
         # names are in order
@@ -196,6 +194,28 @@ class TestLayerConfiguration(object):
         eq_(layers_conf['onea'].conf['name'], 'onea')
         eq_(layers_conf['onea'].conf['sources'], ['s'])
     
+    def test_hierarchy_root_is_list(self):
+        conf = self._test_conf('''
+            layers:
+              - title: Root Layer
+                layers:
+                    - name: one
+                      title: Layer One
+                      sources: [s]
+                    - name: two
+                      title: Layer Two
+                      sources: [s]
+        ''')
+        conf = ProxyConfiguration(conf)
+        root = conf.wms_root_layer.wms_layer()
+        
+        eq_(root.title, 'Root Layer')
+        eq_(root.name, None)
+
+        layers = root.child_layers()
+        # names are in order
+        eq_(layers.keys(), ['one', 'two'])
+    
     def test_without_sources_or_layers(self):
         conf = self._test_conf('''
             layers:
@@ -317,8 +337,8 @@ class TestWMSSourceConfiguration(object):
         
         conf = ProxyConfiguration(conf_dict)
         try:
-            wms = conf.sources['osm'].source({'format': 'image/png'})
-        except ImportError, e:
+            conf.sources['osm'].source({'format': 'image/png'})
+        except ImportError:
             raise SkipTest('no ssl support')
         
     
