@@ -27,9 +27,11 @@ import threading
 import shutil
 import datetime
 import contextlib
-import mapproxy.config
+from functools import wraps
 from copy import deepcopy
 
+import mapproxy.config
+
 import logging
 log = logging.getLogger(__name__)
 
@@ -49,92 +51,7 @@ def local_base_config(conf):
         yield
     finally:
         mapproxy.config.config._config._pop_object()
-    
-class ThreadedExecutor(object):
-    class Executor(threading.Thread):
-        def __init__(self, func, task_queue, result_queue):
-            threading.Thread.__init__(self)
-            self.func = func
-            self.task_queue = task_queue
-            self.result_queue = result_queue
-            self.base_config = deepcopy(mapproxy.config.base_config())
-        def run(self):
-            with local_base_config(self.base_config):
-                while True:
-                    task = self.task_queue.get()
-                    if task is None:
-                        self.task_queue.task_done()
-                        break
-                    exec_id, args = task
-                    try:
-                        result = self.func(args)
-                    except Exception:
-                        result = sys.exc_info()
-                    self.result_queue.put((exec_id, result))
-                    self.task_queue.task_done()
-    def __init__(self, func, pool_size=2):
-        self.func = func
-        self.pool_size = pool_size
-        self.task_queue = Queue.Queue()
-        self.result_queue = Queue.Queue()
-        self.pool = self._init_pool()
-    def execute(self, args):
-        for i, arg in enumerate(args):
-            self.task_queue.put((i, arg))
-        result = []
-        # immediately get results so we can raise exceptions asap 
-        self._get_results(result)
-        # wait for task_queue (all task_done calls)...
-        self.task_queue.join()
-        # and get remaining results
-        self._get_results(result)
-        
-        result.sort()
-        return [value for _, value in result]
-    
-    def _get_results(self, result):
-        while not self.task_queue.empty() or not self.result_queue.empty():
-            task_result = self.result_queue.get()
-            if isinstance(task_result[1], tuple) and \
-               isinstance(task_result[1][1], Exception):
-                self.shutdown(force=True)
-                exc_class, exc, tb = task_result[1]
-                raise exc_class, exc, tb
-            result.append(task_result)
-        
-    def __del__(self):
-        self.shutdown()
-    def shutdown(self, force=False):
-        """
-        Send shutdown sentinel to all executor threads. If `force` is True,
-        clean task_queue and result_queue.
-        """
-        if force:
-            _consume_queue(self.task_queue)
-            _consume_queue(self.result_queue)
-        for _ in range(self.pool_size):
-            self.task_queue.put(None)
-    
-    def _init_pool(self):
-        pool = []
-        for _ in range(self.pool_size):
-            t = ThreadedExecutor.Executor(self.func, self.task_queue, self.result_queue)
-            t.daemon = True
-            t.start()
-            pool.append(t)
-        return pool
 
-def _consume_queue(queue):
-    """
-    Get all items from queue.
-    """
-    while not queue.empty():
-        try:
-            queue.get(block=False)
-            queue.task_done()
-        except Queue.Empty:
-            pass
-    
 def reraise_exception(new_exc, exc_info):
     """
     Reraise exception (`new_exc`) with the given `exc_info`.
@@ -168,6 +85,17 @@ class cached_property(object):
         setattr(obj, self.__name__, value)
         return value
 
+def memoize(func):
+    @wraps(func)
+    def wrapper(*args):
+        if not hasattr(func, '__memoize_cache'):
+            func.__memoize_cache = {}
+        key = args
+        if key not in func.__memoize_cache:
+            func.__memoize_cache[key] = func(*args)
+        return func.__memoize_cache[key]
+    return wrapper
+
 def swap_dir(src_dir, dst_dir, keep_old=False, backup_ext='.tmp'):
     """
     Rename `src_dir` to `dst_dir`. The `dst_dir` is first renamed to
@@ -227,7 +155,10 @@ def timestamp_from_isodate(isodate):
     ...
     ValueError: ...
     """
-    date = datetime.datetime.strptime(isodate, "%Y-%m-%dT%H:%M:%S")
+    if isinstance(isodate, datetime.datetime):
+        date = isodate
+    else:
+        date = datetime.datetime.strptime(isodate, "%Y-%m-%dT%H:%M:%S")
     return time.mktime(date.timetuple())
 
 def cleanup_directory(directory, before_timestamp, remove_empty_dirs=True, 
@@ -243,11 +174,23 @@ def cleanup_directory(directory, before_timestamp, remove_empty_dirs=True,
                 continue
             for filename in filenames:
                 filename = os.path.join(dirpath, filename)
-                if os.lstat(filename).st_mtime < before_timestamp:
-                    file_handler(filename)
-            if (remove_empty_dirs and not os.listdir(dirpath)
-                and dirpath != directory):
-                os.rmdir(dirpath)
+                try:
+                    if os.lstat(filename).st_mtime < before_timestamp:
+                        file_handler(filename)
+                except OSError, ex:
+                    if ex.errno != errno.ENOENT: raise
+
+            if remove_empty_dirs:
+                remove_dir_if_emtpy(dirpath)
+    
+        if remove_empty_dirs:
+            remove_dir_if_emtpy(directory)
+
+def remove_dir_if_emtpy(directory):
+    try:
+        os.rmdir(directory)
+    except OSError, ex:
+        if ex.errno != errno.ENOENT and ex.errno != errno.ENOTEMPTY: raise
 
 def replace_instancemethod(old_method, new_method):
     """
